#import "GBGradientView.h"


@interface GBGradientView ()
@property (nonatomic,strong) NSArray *colors;
@property (nonatomic,strong) NSArray *locationsOne;
@property (nonatomic,strong) NSArray *locationsTwo;
@property (nonatomic,strong) CALayer *imageLayer;
@property (nonatomic,strong) CAGradientLayer *gradientLayer;
@end

@implementation GBGradientView

- (id)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self createScene];
    }
    return self;
}

- (void)createScene {
    // The colors we'll be using for the gradient
    self.colors = @[(id)[UIColor clearColor].CGColor,
                    (id)[UIColor redColor].CGColor,
                    (id)[UIColor clearColor].CGColor];

    // The gradient stops at both ends of the animation
    self.locationsOne = @[@(0.3f), @(0.35f), @(0.4f)];
    self.locationsTwo = @[@(0.6f), @(0.65f), @(0.7f)];

    // Create the barcode image and add it as a sublayer
    self.imageLayer = [CALayer new];
    self.imageLayer.contents = (id)[UIImage imageNamed:@"barcode"].CGImage;
    self.imageLayer.contentsGravity = kCAGravityCenter;
    self.imageLayer.contentsScale = [UIScreen mainScreen].scale;
    [self.layer addSublayer:self.imageLayer];

    // Create the gradient layer and add it as a sublayer
    self.gradientLayer = [CAGradientLayer new];
    self.gradientLayer.opacity = 0.5;
    self.gradientLayer.colors = self.colors;
    self.gradientLayer.locations = self.locationsOne;
    [self.layer addSublayer:self.gradientLayer];
}

- (void)layoutSublayersOfLayer:(CALayer *)layer {
    if (layer != self.layer) return;

    self.imageLayer.frame = self.bounds;
    self.gradientLayer.frame = self.bounds;
}

- (void)startAnimations{
    // Make the laser beam move up and down
    CABasicAnimation* flowAnimation = [CABasicAnimation
                                          animationWithKeyPath:@"locations"];
    flowAnimation.fromValue = self.locationsOne;
    flowAnimation.toValue = self.locationsTwo;
    flowAnimation.duration = 1;
    flowAnimation.repeatCount = INFINITY;
    flowAnimation.autoreverses = YES;
    [self.gradientLayer addAnimation:flowAnimation
                          forKey:@"flowAnimation"];
}

- (void)stopAnimations{
    [self.gradientLayer removeAllAnimations];
}

@end
