#import "GBMaskingView.h"

#define SET_SHADOW_PATHS YES


@interface GBMaskingView ()
@property (nonatomic,strong) CAShapeLayer *roundedRectLayer;
@property (nonatomic,strong) CAShapeLayer *ovalLayer;
@end

@implementation GBMaskingView

- (id)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self createScene];
    }
    return self;
}

- (void)createScene {
    // Create a CAShapeLayer with a yellow rounded rect shape, and add it to the view
    self.roundedRectLayer = [CAShapeLayer new];
    self.roundedRectLayer.fillColor = [UIColor yellowColor].CGColor;
    [self.layer addSublayer:self.roundedRectLayer];

    // Give the shape a shadow
    self.roundedRectLayer.shadowColor = [UIColor blackColor].CGColor;
    self.roundedRectLayer.shadowOpacity = 0.5;
    self.roundedRectLayer.shadowRadius = 15;

    // Create a CAShapeLayer with an oval shape
    self.ovalLayer = [CAShapeLayer new];

    // Give the oval layer a "shadow", which will create the illusion of a spotlight
    self.ovalLayer.shadowColor = [UIColor blackColor].CGColor;
    self.ovalLayer.shadowOpacity = 1.0;
    self.ovalLayer.shadowRadius = 40;
    
    // Set the oval as a mask for the rounded rect
    self.layer.mask = self.ovalLayer;
}

- (void)layoutSublayersOfLayer:(CALayer *)layer {
    if (layer != self.layer) return;

    // Layout rounded rect
    self.roundedRectLayer.frame = self.bounds;
    self.roundedRectLayer.path = [GBShapes
                                  roundedRectShapeInFrame:CGRectInset(self.bounds, 40, 40)].CGPath;
    if( SET_SHADOW_PATHS ){
        self.roundedRectLayer.shadowPath = self.roundedRectLayer.path;
    }

    // Layout oval layer
    self.ovalLayer.frame = self.bounds;
    self.ovalLayer.path = [GBShapes ovalShapeInFrame:CGRectInset(self.bounds, 60, 60)].CGPath;
    if( SET_SHADOW_PATHS ){
        self.ovalLayer.shadowPath = self.ovalLayer.path;
    }

    // Position the oval layer
    self.ovalLayer.position = CGPointMake(self.ovalLayer.position.x, self.ovalLayer.position.y + floorf(self.bounds.size.height/2.0));
    self.ovalLayer.anchorPoint = CGPointMake(0.5, 1);
}

- (void)startAnimations {
    // Make the circle move around like we're shining a flashlight
    CABasicAnimation* oscillationAnimation = [CABasicAnimation
                                              animationWithKeyPath:@"transform"];
    oscillationAnimation.fromValue = (id)[NSValue valueWithCATransform3D:CATransform3DMakeRotation(-M_PI_4, 0.5, 0, 1)];
    oscillationAnimation.toValue = (id)[NSValue valueWithCATransform3D:CATransform3DMakeRotation(M_PI_4, 0.5, 0, 1)];
    oscillationAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
    oscillationAnimation.duration = 1.0;
    oscillationAnimation.autoreverses = YES;
    oscillationAnimation.repeatCount = INFINITY;
    [self.ovalLayer addAnimation:oscillationAnimation
                          forKey:@"oscillationAnimation"];

    // Make the light flicker a bit
    CABasicAnimation* flickerAnimation = [CABasicAnimation
                                              animationWithKeyPath:@"opacity"];
    flickerAnimation.fromValue = (id)[NSNumber numberWithFloat:0.6];
    flickerAnimation.toValue = (id)[NSNumber numberWithFloat:0.7];
    flickerAnimation.duration = 0.01;
    flickerAnimation.autoreverses = YES;
    flickerAnimation.repeatCount = INFINITY;
    [self.ovalLayer addAnimation:flickerAnimation
                          forKey:@"flickerAnimation"];
}

- (void)stopAnimations {
    [self.ovalLayer removeAllAnimations];
}

@end
