#import "GBRotationView.h"

@interface GBRotationView ()
@property (nonatomic,strong) CALayer *earthImageLayer;
@property (nonatomic,strong) CALayer *moonImageLayer;
@property (nonatomic,strong) CAEmitterLayer *starsEmitter;
@end

@implementation GBRotationView

- (id)initWithCoder:(NSCoder *)aDecoder {
    self = [super initWithCoder:aDecoder];
    if (self) {
        [self createScene];
    }
    return self;
}

- (void)createScene {
    // Create a particle emitter for the "stars"
    self.starsEmitter  = [CAEmitterLayer new];
    self.starsEmitter.emitterShape = kCAEmitterLayerRectangle;
    self.starsEmitter.emitterPosition = self.center;
    self.starsEmitter.emitterSize = self.bounds.size;
    self.starsEmitter.emitterMode = kCAEmitterLayerRectangle;
    self.starsEmitter.renderMode = kCAEmitterLayerSurface;
    [self.layer addSublayer:self.starsEmitter];

    CAEmitterCell* star = [CAEmitterCell emitterCell];
    star.contents = (id)[[UIImage imageNamed:@"star"] CGImage];
    star.birthRate = 20;
    star.lifetime = 5.0;
    star.lifetimeRange = 3.0;
    star.scale = 0.05;
    star.scaleRange = 0.17;
    [star setName:@"star"];

    //add the cell to the layer and we're done
    self.starsEmitter.emitterCells = [NSArray arrayWithObject:star];

    // Create the earth image and add it as a sublayer
    self.earthImageLayer = [CALayer new];
    self.earthImageLayer.contents = (id)[UIImage imageNamed:@"earth"].CGImage;
    self.earthImageLayer.contentsGravity = kCAGravityCenter;
    self.earthImageLayer.contentsScale = [UIScreen mainScreen].scale;
    [self.layer addSublayer:self.earthImageLayer];

    // Create the moon image and add it as a sublayer
    self.moonImageLayer = [CALayer new];
    self.moonImageLayer.contents = (id)[UIImage imageNamed:@"moon"].CGImage;
    self.moonImageLayer.contentsGravity = kCAGravityRight;
    self.moonImageLayer.contentsScale = [UIScreen mainScreen].scale;
    [self.layer addSublayer:self.moonImageLayer];
}

- (void)layoutSublayersOfLayer:(CALayer *)layer {
    if (layer != self.layer) return;

    self.earthImageLayer.frame = self.bounds;
    self.moonImageLayer.frame = self.bounds;
    self.starsEmitter.frame = self.bounds;
}

- (void)startAnimations {
    // Make the earth spin, slowly
    CABasicAnimation* earthRotationAnimation;
    earthRotationAnimation = [CABasicAnimation animationWithKeyPath:@"transform.rotation.z"];
    earthRotationAnimation.toValue = [NSNumber numberWithFloat:M_PI * 2.0];
    earthRotationAnimation.duration = 10;
    earthRotationAnimation.repeatCount = INFINITY;
    [self.earthImageLayer addAnimation:earthRotationAnimation forKey:@"rotationAnimation"];

    // Make the moon spin, even slower
    CABasicAnimation* moonRotationAnimation;
    moonRotationAnimation = [CABasicAnimation animationWithKeyPath:@"transform.rotation.z"];
    moonRotationAnimation.toValue = [NSNumber numberWithFloat:-M_PI * 2.0];
    moonRotationAnimation.duration = 15;
    moonRotationAnimation.repeatCount = INFINITY;
    [self.moonImageLayer addAnimation:moonRotationAnimation forKey:@"rotationAnimation"];

    // Make the stars flicker a bit
    CABasicAnimation* flickerAnimation = [CABasicAnimation
                                          animationWithKeyPath:@"opacity"];
    flickerAnimation.fromValue = (id)[NSNumber numberWithFloat:0.8];
    flickerAnimation.toValue = (id)[NSNumber numberWithFloat:1];
    flickerAnimation.duration = 0.01;
    flickerAnimation.autoreverses = YES;
    flickerAnimation.repeatCount = INFINITY;
    [self.starsEmitter addAnimation:flickerAnimation
                          forKey:@"flickerAnimation"];
}

- (void)stopAnimations {
    [self.earthImageLayer removeAllAnimations];
    [self.moonImageLayer removeAllAnimations];
    [self.starsEmitter removeAllAnimations];
}

@end
